@@ -1,12 +1,11 @@
 import io
 import os.path
 import pickle
+import tempfile
 
-from dagster_databricks import DatabricksJobRunner, databricks_step_main
-from dagster_pyspark.utils import build_pyspark_zip
-
-from dagster import Bool, Field, StringSource, check, resource, seven
+from dagster import Bool, Field, IntSource, StringSource, check, resource
 from dagster.core.definitions.step_launcher import StepLauncher
+from dagster.core.errors import raise_execution_interrupts
 from dagster.core.events import log_step_event
 from dagster.core.execution.plan.external_step import (
     PICKLED_EVENTS_FILE_NAME,
@@ -14,6 +13,9 @@ from dagster.core.execution.plan.external_step import (
     step_context_to_step_run_ref,
 )
 from dagster.serdes import deserialize_value
+from dagster_databricks import databricks_step_main
+from dagster_databricks.databricks import DEFAULT_RUN_MAX_WAIT_TIME_SEC, DatabricksJobRunner
+from dagster_pyspark.utils import build_pyspark_zip
 
 from .configs import (
     define_databricks_secrets_config,
@@ -34,7 +36,9 @@ PICKLED_CONFIG_FILE_NAME = "config.pkl"
             description="Databricks host, e.g. uksouth.azuredatabricks.com",
         ),
         "databricks_token": Field(
-            StringSource, is_required=True, description="Databricks access token",
+            StringSource,
+            is_required=True,
+            description="Databricks access token",
         ),
         "secrets_to_env_variables": define_databricks_secrets_config(),
         "storage": define_databricks_storage_config(),
@@ -63,6 +67,13 @@ PICKLED_CONFIG_FILE_NAME = "config.pkl"
             "location. Note that logs are copied every 5 minutes, so enabling this will add "
             "several minutes to the job runtime.",
         ),
+        "max_completion_wait_time_seconds": Field(
+            IntSource,
+            is_required=False,
+            default_value=DEFAULT_RUN_MAX_WAIT_TIME_SEC,
+            description="If the Databricks job run takes more than this many seconds, then "
+            "consider it failed and terminate the step.",
+        ),
     }
 )
 def databricks_pyspark_step_launcher(context):
@@ -93,6 +104,7 @@ class DatabricksPySparkStepLauncher(StepLauncher):
         local_pipeline_package_path,
         staging_prefix,
         wait_for_logs,
+        max_completion_wait_time_seconds,
     ):
         self.run_config = check.dict_param(run_config, "run_config")
         self.databricks_host = check.str_param(databricks_host, "databricks_host")
@@ -106,7 +118,11 @@ class DatabricksPySparkStepLauncher(StepLauncher):
         check.invariant(staging_prefix.startswith("/"), "staging_prefix must be an absolute path")
         self.wait_for_logs = check.bool_param(wait_for_logs, "wait_for_logs")
 
-        self.databricks_runner = DatabricksJobRunner(host=databricks_host, token=databricks_token)
+        self.databricks_runner = DatabricksJobRunner(
+            host=databricks_host,
+            token=databricks_token,
+            max_wait_time_sec=max_completion_wait_time_seconds,
+        )
 
     def launch_step(self, step_context, prior_attempts_count):
         step_run_ref = step_context_to_step_run_ref(
@@ -122,7 +138,10 @@ class DatabricksPySparkStepLauncher(StepLauncher):
         databricks_run_id = self.databricks_runner.submit_run(self.run_config, task)
 
         try:
-            self.databricks_runner.wait_for_run_to_complete(log, databricks_run_id)
+            # If this is being called within a `capture_interrupts` context, allow interrupts while
+            # waiting for the  execution to complete, so that we can terminate slow or hanging steps
+            with raise_execution_interrupts():
+                self.databricks_runner.wait_for_run_to_complete(log, databricks_run_id)
         finally:
             if self.wait_for_logs:
                 self._log_logs_from_cluster(log, databricks_run_id)
@@ -164,7 +183,7 @@ class DatabricksPySparkStepLauncher(StepLauncher):
             )
 
         log.info("Uploading pipeline to DBFS")
-        with seven.TemporaryDirectory() as temp_dir:
+        with tempfile.TemporaryDirectory() as temp_dir:
             # Zip and upload package containing pipeline
             zip_local_path = os.path.join(temp_dir, CODE_ZIP_NAME)
             build_pyspark_zip(zip_local_path, self.local_pipeline_package_path)
@@ -179,17 +198,22 @@ class DatabricksPySparkStepLauncher(StepLauncher):
         pickle.dump(step_run_ref, step_pickle_file)
         step_pickle_file.seek(0)
         self.databricks_runner.client.put_file(
-            step_pickle_file, self._dbfs_path(run_id, step_key, PICKLED_STEP_RUN_REF_FILE_NAME),
+            step_pickle_file,
+            self._dbfs_path(run_id, step_key, PICKLED_STEP_RUN_REF_FILE_NAME),
         )
 
-        databricks_config = DatabricksConfig(storage=self.storage, secrets=self.secrets,)
+        databricks_config = DatabricksConfig(
+            storage=self.storage,
+            secrets=self.secrets,
+        )
         log.info("Uploading Databricks configuration to DBFS")
         databricks_config_file = io.BytesIO()
 
         pickle.dump(databricks_config, databricks_config_file)
         databricks_config_file.seek(0)
         self.databricks_runner.client.put_file(
-            databricks_config_file, self._dbfs_path(run_id, step_key, PICKLED_CONFIG_FILE_NAME),
+            databricks_config_file,
+            self._dbfs_path(run_id, step_key, PICKLED_CONFIG_FILE_NAME),
         )
 
     def _log_logs_from_cluster(self, log, run_id):
